@@ -13,7 +13,7 @@ With no report interval (@--monthly@ etc.), hledger's balance
 command emulates ledger's, showing accounts indented according to
 hierarchy, along with their total amount posted (including subaccounts).
 
-Here's an example. With @data/sample.journal@, which defines the following account tree:
+Here's an example. With @examples/sample.journal@, which defines the following account tree:
 
 @
  assets
@@ -239,16 +239,15 @@ module Hledger.Cli.Balance (
  ,balance
  ,balanceReportAsText
  ,balanceReportItemAsText
- ,periodChangeReportAsText
- ,cumulativeChangeReportAsText
- ,historicalBalanceReportAsText
+ ,multiBalanceReportAsText
+ ,renderBalanceReportTable
+ ,balanceReportAsTable
  ,tests_Hledger_Cli_Balance
 ) where
 
 import Data.List (intercalate)
 import Data.Maybe
-import Data.Monoid
--- import Data.Text (Text)
+-- import Data.Monoid
 import qualified Data.Text as T
 import System.Console.CmdArgs.Explicit as C
 import Text.CSV
@@ -275,13 +274,13 @@ balancemode = (defCommandMode $ ["balance"] ++ aliases) { -- also accept but don
         "show historical ending balance in each period (includes postings before report start date)\n "
      ,flagNone ["tree"] (\opts -> setboolopt "tree" opts) "show accounts as a tree; amounts include subaccounts (default in simple reports)"
      ,flagNone ["flat"] (\opts -> setboolopt "flat" opts) "show accounts as a list; amounts exclude subaccounts except when account is depth-clipped (default in multicolumn reports)\n "
-     ,flagNone ["value","V"] (setboolopt "value") "convert amounts to their market value on the report end date (using the most recent applicable market price, if any)"
      ,flagNone ["average","A"] (\opts -> setboolopt "average" opts) "show a row average column (in multicolumn reports)"
      ,flagNone ["row-total","T"] (\opts -> setboolopt "row-total" opts) "show a row total column (in multicolumn reports)"
      ,flagNone ["no-total","N"] (\opts -> setboolopt "no-total" opts) "omit the final total row"
      ,flagReq  ["drop"] (\s opts -> Right $ setopt "drop" s opts) "N" "omit N leading account name parts (in flat mode)"
      ,flagNone ["no-elide"] (\opts -> setboolopt "no-elide" opts) "don't squash boring parent accounts (in tree mode)"
      ,flagReq  ["format"] (\s opts -> Right $ setopt "format" s opts) "FORMATSTR" "use this custom line format (in simple reports)"
+     ,flagNone ["pretty-tables"] (\opts -> setboolopt "pretty-tables" opts) "use unicode when displaying tables"
      ]
      ++ outputflags
     ,groupHidden = []
@@ -299,8 +298,6 @@ balance opts@CliOpts{reportopts_=ropts} j = do
     Right _ -> do
       let format   = outputFormatFromOpts opts
           interval = interval_ ropts
-          baltype  = balancetype_ ropts
-      mvaluedate <- reportEndDate j ropts
       -- shenanigans: use single/multiBalanceReport when we must,
       -- ie when there's a report interval, or --historical or -- cumulative.
       -- Otherwise prefer the older balanceReport since it can elide boring parents.
@@ -314,23 +311,16 @@ balance opts@CliOpts{reportopts_=ropts} j = do
                                | otherwise   = ropts{accountlistmode_=ALTree}
                     in singleBalanceReport ropts' (queryFromOpts d ropts) j
                 | otherwise = balanceReport ropts (queryFromOpts d ropts) j
-              convert | value_ ropts = maybe id (balanceReportValue j) mvaluedate
-                      | otherwise = id
               render = case format of
                 "csv" -> \ropts r -> (++ "\n") $ printCSV $ balanceReportAsCsv ropts r
                 _     -> balanceReportAsText
-          writeOutput opts $ render ropts $ convert report
+          writeOutput opts $ render ropts report
         _ -> do
           let report = multiBalanceReport ropts (queryFromOpts d ropts) j
-              convert | value_ ropts = maybe id (multiBalanceReportValue j) mvaluedate
-                      | otherwise = id
               render = case format of
                 "csv" -> \ropts r -> (++ "\n") $ printCSV $ multiBalanceReportAsCsv ropts r
-                _     -> case baltype of
-                  PeriodChange     -> periodChangeReportAsText
-                  CumulativeChange -> cumulativeChangeReportAsText
-                  HistoricalBalance -> historicalBalanceReportAsText
-          writeOutput opts $ render ropts $ convert report
+                _     -> multiBalanceReportAsText
+          writeOutput opts $ render ropts report
 
 -- single-column balance reports
 
@@ -481,94 +471,52 @@ multiBalanceReportAsCsv opts (MultiBalanceReport (colspans, items, (coltotals,to
            ++ (if average_ opts then [avg] else [])
            )]
 
--- | Render a multi-column period balance report as plain text suitable for console output.
-periodChangeReportAsText :: ReportOpts -> MultiBalanceReport -> String
-periodChangeReportAsText opts r@(MultiBalanceReport (colspans, items, (coltotals,tot,avg))) =
-  unlines $
-  ([printf "Balance changes in %s:" (showDateSpan $ multiBalanceReportSpan r)] ++) $
-  trimborder $ lines $
-   render id (" "++) showMixedAmountOneLineWithoutPrice $
-    addtotalrow $
-     Table
-     (T.Group NoLine $ map (Header . padRightWide acctswidth . T.unpack) accts)
+-- | Render a multi-column balance report as plain text suitable for console output.
+multiBalanceReportAsText :: ReportOpts -> MultiBalanceReport -> String
+multiBalanceReportAsText opts r =
+    printf "%s in %s:" typeStr (showDateSpan $ multiBalanceReportSpan r)
+      ++ "\n"
+      ++ renderBalanceReportTable opts tabl
+  where
+    tabl = balanceReportAsTable opts r
+    typeStr :: String
+    typeStr = case balancetype_ opts of
+        PeriodChange -> "Balance changes"
+        CumulativeChange -> "Ending balances (cumulative)"
+        HistoricalBalance -> "Ending balances (historical)"
+
+-- | Given a table representing a multi-column balance report (for example,
+-- made using 'balanceReportAsTable'), render it in a format suitable for
+-- console output.
+renderBalanceReportTable :: ReportOpts -> Table String String MixedAmount -> String
+renderBalanceReportTable (ReportOpts { pretty_tables_ = pretty }) = unlines . trimborder . lines
+                         . render pretty id (" " ++) showMixedAmountOneLineWithoutPrice
+                         . align
+  where
+    trimborder = ("":) . (++[""]) . drop 1 . init . map (drop 1 . init)
+    align (Table l t d) = Table l' t d
+      where
+        acctswidth = maximum' $ map strWidth (headerContents l)
+        l'         = padRightWide acctswidth <$> l
+
+-- | Build a 'Table' from a multi-column balance report.
+balanceReportAsTable :: ReportOpts -> MultiBalanceReport -> Table String String MixedAmount
+balanceReportAsTable opts (MultiBalanceReport (colspans, items, (coltotals,tot,avg))) =
+   addtotalrow $ Table
+     (T.Group NoLine $ map Header accts)
      (T.Group NoLine $ map Header colheadings)
-     (map rowvals items')
+     (map rowvals items)
   where
-    trimborder = ("":) . (++[""]) . drop 1 . init . map (drop 1 . init)
-    colheadings = map showDateSpan colspans
-                  ++ (if row_total_ opts then ["  Total"] else [])
-                  ++ (if average_ opts then ["Average"] else [])
-    items' | empty_ opts = items
-           | otherwise   = items -- dbg1 "2" $ filter (any (not . isZeroMixedAmount) . snd) $ dbg1 "1" items
-    accts = map renderacct items'
-    renderacct (a,a',i,_,_,_)
-      | tree_ opts = T.replicate ((i-1)*2) " " <> a'
-      | otherwise  = maybeAccountNameDrop opts a
-    acctswidth = maximum' $ map textWidth accts
-    rowvals (_,_,_,as,rowtot,rowavg) = as
-                                   ++ (if row_total_ opts then [rowtot] else [])
-                                   ++ (if average_ opts then [rowavg] else [])
-    addtotalrow | no_total_ opts = id
-                | otherwise      = (+----+ (row "" $
-                                    coltotals
-                                    ++ (if row_total_ opts then [tot] else [])
-                                    ++ (if average_ opts then [avg] else [])
-                                    ))
-
--- | Render a multi-column cumulative balance report as plain text suitable for console output.
-cumulativeChangeReportAsText :: ReportOpts -> MultiBalanceReport -> String
-cumulativeChangeReportAsText opts r@(MultiBalanceReport (colspans, items, (coltotals,tot,avg))) =
-  unlines $
-  ([printf "Ending balances (cumulative) in %s:" (showDateSpan $ multiBalanceReportSpan r)] ++) $
-  trimborder $ lines $
-   render id (" "++) showMixedAmountOneLineWithoutPrice $
-    addtotalrow $
-     Table
-       (T.Group NoLine $ map (Header . padRightWide acctswidth) accts)
-       (T.Group NoLine $ map Header colheadings)
-       (map rowvals items)
-  where
-    trimborder = ("":) . (++[""]) . drop 1 . init . map (drop 1 . init)
-    colheadings = map (maybe "" (showDate . prevday) . spanEnd) colspans
+    mkDate = case balancetype_ opts of
+       PeriodChange -> showDateSpan
+       _            -> maybe "" (showDate . prevday) . spanEnd
+    colheadings = map mkDate colspans
                   ++ (if row_total_ opts then ["  Total"] else [])
                   ++ (if average_ opts then ["Average"] else [])
     accts = map renderacct items
     renderacct (a,a',i,_,_,_)
       | tree_ opts = replicate ((i-1)*2) ' ' ++ T.unpack a'
       | otherwise  = T.unpack $ maybeAccountNameDrop opts a
-    acctswidth = maximum' $ map strWidth accts
-    rowvals (_,_,_,as,rowtot,rowavg) = as
-                                   ++ (if row_total_ opts then [rowtot] else [])
-                                   ++ (if average_ opts then [rowavg] else [])
-    addtotalrow | no_total_ opts = id
-                | otherwise      = (+----+ (row "" $
-                                    coltotals
-                                    ++ (if row_total_ opts then [tot] else [])
-                                    ++ (if average_ opts then [avg] else [])
-                                    ))
-
--- | Render a multi-column historical balance report as plain text suitable for console output.
-historicalBalanceReportAsText :: ReportOpts -> MultiBalanceReport -> String
-historicalBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, (coltotals,tot,avg))) =
-  unlines $
-  ([printf "Ending balances (historical) in %s:" (showDateSpan $ multiBalanceReportSpan r)] ++) $
-  trimborder $ lines $
-   render id (" "++) showMixedAmountOneLineWithoutPrice $
-    addtotalrow $
-     Table
-       (T.Group NoLine $ map (Header . padRightWide acctswidth) accts)
-       (T.Group NoLine $ map Header colheadings)
-       (map rowvals items)
-  where
-    trimborder = ("":) . (++[""]) . drop 1 . init . map (drop 1 . init)
-    colheadings = map (maybe "" (showDate . prevday) . spanEnd) colspans
-                  ++ (if row_total_ opts then ["  Total"] else [])
-                  ++ (if average_ opts then ["Average"] else [])
-    accts = map renderacct items
-    renderacct (a,a',i,_,_,_)
-      | tree_ opts = replicate ((i-1)*2) ' ' ++ T.unpack a'
-      | otherwise  = T.unpack $ maybeAccountNameDrop opts a
-    acctswidth = maximum' $ map strWidth accts
     rowvals (_,_,_,as,rowtot,rowavg) = as
                              ++ (if row_total_ opts then [rowtot] else [])
                              ++ (if average_ opts then [rowavg] else [])
