@@ -20,7 +20,8 @@ module Hledger.Cli.CliOptions (
   generalflagsgroup3,
   defMode,
   defCommandMode,
-  defAddonCommandMode,
+  quickAddonCommandMode,
+  hledgerCommandMode,
   argsFlag,
   showModeUsage,
   withAliases,
@@ -28,7 +29,7 @@ module Hledger.Cli.CliOptions (
   -- * CLI options
   CliOpts(..),
   defcliopts,
-  getCliOpts,
+  getHledgerCliOpts,
   decodeRawOpts,
   rawOptsToCliOpts,
   checkCliOpts,
@@ -73,6 +74,7 @@ import Data.Functor.Compat ((<$>))
 import Data.Functor.Identity (Identity)
 import Data.List.Compat
 import Data.List.Split (splitOneOf)
+import Data.Ord
 import Data.Maybe
 -- import Data.Text (Text)
 import qualified Data.Text as T
@@ -100,10 +102,10 @@ import Hledger.Cli.Version
 -- | Common help flags: --help, --debug, --version...
 helpflags :: [Flag RawOpts]
 helpflags = [
-  flagNone ["h"]    (setboolopt "h")    "show general usage or (after COMMAND, the command's usage"
- ,flagNone ["help"] (setboolopt "help") "show the current program's manual as plain text (or after an add-on COMMAND, the add-on's manual)"
- ,flagNone ["man"]  (setboolopt "man")  "show the current program's manual with man"
- ,flagNone ["info"] (setboolopt "info") "show the current program's manual with info"
+  flagNone ["h"]    (setboolopt "h")    "show general usage (or after CMD, command usage)"
+ ,flagNone ["help"] (setboolopt "help") "show this program's manual as plain text (or after an addon CMD, the add-on's manual)"
+ ,flagNone ["man"]  (setboolopt "man")  "show this program's manual with man"
+ ,flagNone ["info"] (setboolopt "info") "show this program's manual with info"
  -- ,flagNone ["browse-args"] (setboolopt "browse-args") "use a web UI to select options and build up a command line"
  ,flagReq  ["debug"]    (\s opts -> Right $ setopt "debug" s opts) "[N]" "show debug output (levels 1-9, default: 1)"
  ,flagNone ["version"] (setboolopt "version") "show version information"
@@ -116,11 +118,12 @@ detailedversionflag = flagNone ["version+"] (setboolopt "version+") "show versio
 -- | Common input-related flags: --file, --rules-file, --alias...
 inputflags :: [Flag RawOpts]
 inputflags = [
-  flagReq ["file","f"]  (\s opts -> Right $ setopt "file" s opts) "FILE" "use a different input file. For stdin, use -"
- ,flagReq ["rules-file"]  (\s opts -> Right $ setopt "rules-file" s opts) "RFILE" "CSV conversion rules file (default: FILE.rules)"
- ,flagReq ["alias"]  (\s opts -> Right $ setopt "alias" s opts)  "OLD=NEW" "display accounts named OLD as NEW"
- ,flagNone ["ignore-assertions","I"] (setboolopt "ignore-assertions") "ignore any balance assertions in the journal"
- ,flagReq ["pivot"]  (\s opts -> Right $ setopt "pivot" s opts)  "TAG" "Replace the accounts of postings with TAG by TAG:<value>"
+  flagReq  ["file","f"]      (\s opts -> Right $ setopt "file" s opts) "FILE" "use a different input file. For stdin, use - (default: $LEDGER_FILE or $HOME/.hledger.journal)"
+ ,flagReq  ["rules-file"]    (\s opts -> Right $ setopt "rules-file" s opts) "RFILE" "CSV conversion rules file (default: FILE.rules)"
+ ,flagReq  ["alias"]         (\s opts -> Right $ setopt "alias" s opts)  "OLD=NEW" "rename accounts named OLD to NEW"
+ ,flagNone ["anon"]          (setboolopt "anon") "anonymize accounts and payees"
+ ,flagReq  ["pivot"]         (\s opts -> Right $ setopt "pivot" s opts)  "TAGNAME" "use some other field/tag for account names"
+ ,flagNone ["ignore-assertions","I"] (setboolopt "ignore-assertions") "ignore any balance assertions"
  ]
 
 -- | Common report-related flags: --period, --cost, etc.
@@ -143,7 +146,7 @@ reportflags = [
  ,flagReq  ["depth"]         (\s opts -> Right $ setopt "depth" s opts) "N" "hide accounts/postings deeper than N"
  ,flagNone ["empty","E"]     (setboolopt "empty") "show items with zero amount, normally hidden"
  ,flagNone ["cost","B"]      (setboolopt "cost") "convert amounts to their cost at transaction time (using the transaction price, if any)"
- ,flagNone ["anon"]              (setboolopt "anon") "output ledger with anonymized accounts and payees."
+ ,flagNone ["value","V"]     (setboolopt "value") "convert amounts to their market value on the report end date (using the most recent applicable market price, if any)"
  ]
 
 -- | Common output-related flags: --output-file, --output-format...
@@ -165,51 +168,107 @@ generalflagsgroup3 = (generalflagstitle, helpflags)
 
 -- cmdargs mode constructors
 
--- | A basic mode template.
+-- | An empty cmdargs mode to use as a template.
+-- Modes describe the top-level command, ie the program, or a subcommand,
+-- telling cmdargs how to parse a command line and how to
+-- generate the command's usage text.
 defMode :: Mode RawOpts
-defMode =   Mode {
-  modeNames = []
- ,modeHelp = ""
- ,modeHelpSuffix = []
- ,modeValue = []
- ,modeCheck = Right
- ,modeReform = const Nothing
- ,modeExpandAt = True
- ,modeGroupFlags = Group {
-     groupNamed = []
-    ,groupUnnamed = [
-        flagNone ["h"] (setboolopt "h") "Show command usage."
-        ]
-    ,groupHidden = []
-    }
- ,modeArgs = ([], Nothing)
- ,modeGroupModes = toGroup []
+defMode = Mode {
+  modeNames       = []            -- program/command name(s)
+ ,modeHelp        = ""            -- short help for this command
+ ,modeHelpSuffix  = []            -- text displayed after the usage
+ ,modeGroupFlags  = Group {       -- description of flags accepted by the command
+    groupNamed   = []             --  named groups of flags
+   ,groupUnnamed = []             --  ungrouped flags
+   ,groupHidden  = []             --  flags not displayed in the usage
+   }
+ ,modeArgs        = ([], Nothing) -- description of arguments accepted by the command
+ ,modeValue       = []            -- value returned when this mode is used to parse a command line
+ ,modeCheck       = Right         -- whether the mode's value is correct
+ ,modeReform      = const Nothing -- function to convert the value back to a command line arguments
+ ,modeExpandAt    = True          -- expand @ arguments for program ?
+ ,modeGroupModes  = toGroup []    -- sub-modes
  }
 
--- | A basic subcommand mode with the given command name(s).
+-- | A cmdargs mode suitable for a hledger built-in command
+-- with the given names (primary name + optional aliases).
+-- The default flags are short and long help (-h and --help).
+-- The usage message shows [QUERY] as argument.
 defCommandMode :: [Name] -> Mode RawOpts
 defCommandMode names = defMode {
    modeNames=names
+  ,modeGroupFlags  = Group {
+     groupNamed   = []
+    ,groupUnnamed = [
+       flagNone ["h"] (setboolopt "h") "Show usage."
+      -- ,flagNone ["help"] (setboolopt "help") "Show long help."
+      ]
+    ,groupHidden  = []             --  flags not displayed in the usage
+    }
+  ,modeArgs = ([], Just $ argsFlag "[QUERY]")
   ,modeValue=[("command", headDef "" names)]
-  ,modeArgs = ([], Just $ argsFlag "[PATTERNS]")
   }
 
--- | A basic subcommand mode suitable for an add-on command.
-defAddonCommandMode :: Name -> Mode RawOpts
-defAddonCommandMode addon = defMode {
-   modeNames = [addon]
-  ,modeHelp = fromMaybe "" $ lookup (stripAddonExtension addon) standardAddonsHelp
-  ,modeValue=[("command",addon)]
+-- | A cmdargs mode representing the hledger add-on command with the given name.
+-- Like defCommandMode, but adds a appropriate short help message if the addon name
+-- is recognised, and includes hledger's common input/reporting/help flags as default.
+-- Just used by hledger for generating the commands list I think (or possibly for
+-- invoking the addons as well ?)
+quickAddonCommandMode :: Name -> Mode RawOpts
+quickAddonCommandMode name = (defCommandMode [name]) {
+   modeHelp = fromMaybe "" $ lookup (stripAddonExtension name) standardAddonsHelp
   ,modeGroupFlags = Group {
       groupUnnamed = []
      ,groupHidden = []
      ,groupNamed = [generalflagsgroup1]
      }
-  ,modeArgs = ([], Just $ argsFlag "[ARGS]")
   }
 
--- | Built-in descriptions for some of the known external addons,
--- since we don't currently have any way to ask them.
+-- | A template for a command's CLI help, influencing the content and layout
+-- of the usage text generated by a cmdargs mode. 
+-- It is a multiline string structured like so:
+-- The first line defines the command name (first word) and aliases (any other words). 
+-- From the second line up to a line containing just "FLAGS", or the end, is the preamble,
+-- displayed above the flags list generated by cmdargs. Short help goes here.
+-- Any lines after the FLAGS line are the postamble, displayed below the flags list.
+-- Long help/full manual goes here.
+type HelpTemplate = String
+
+-- | Parse a help template into command names, help preamble, and help postamble lines.
+parseHelpTemplate :: HelpTemplate -> Maybe ([Name], String, [String])
+parseHelpTemplate t =
+  case lines t of
+    [] -> Nothing
+    (l:ls) -> Just (names, preamble, postamblelines)
+      where
+        names = words l
+        (preamblels, postamblels) = break (== "FLAGS") ls
+        preamble = unlines $ reverse $ dropWhile null $ reverse preamblels
+        postamblelines = dropWhile null $ drop 1 postamblels
+
+-- | Build a cmdarg mode suitable for a hledger add-on command,
+-- from a help template and flag/argument specifications.
+-- Reduces boilerplate a little, though the complicated cmdargs
+-- flag and argument specs are still required.
+-- See the addons in bin/ for examples of usage.
+hledgerCommandMode :: HelpTemplate -> [Flag RawOpts] -> [(Help, [Flag RawOpts])] 
+  -> [Flag RawOpts] -> ([Arg RawOpts], Maybe (Arg RawOpts)) -> Mode RawOpts
+hledgerCommandMode tmpl ungroupedflags groupedflags hiddenflags args =
+  case parseHelpTemplate tmpl of
+    Nothing -> error' $ "Could not parse help template:\n"++tmpl++"\n"
+    Just (names, preamble, postamblelines) ->
+      (defCommandMode names) {
+         modeHelp = preamble
+        ,modeHelpSuffix = postamblelines
+        ,modeGroupFlags = Group {
+            groupUnnamed = ungroupedflags
+           ,groupNamed   = groupedflags
+           ,groupHidden  = hiddenflags
+           }
+        ,modeArgs = args
+        }
+
+-- | Built-in descriptions for some of the known addons.
 standardAddonsHelp :: [(String,String)]
 standardAddonsHelp = [
    ("chart", "generate simple balance pie charts")
@@ -353,28 +412,53 @@ rawOptsToCliOpts rawopts = checkCliOpts <$> do
 -- | Do final validation of processed opts, raising an error if there is trouble.
 checkCliOpts :: CliOpts -> CliOpts
 checkCliOpts opts =
-  either optserror (const opts) $ do
+  either usageError (const opts) $ do
     -- XXX move to checkReportOpts or move _format to CliOpts
     case lineFormatFromOpts $ reportopts_ opts of
       Left err -> Left $ "could not parse format option: "++err
       Right _  -> Right ()
   -- XXX check registerWidthsFromOpts opts
 
--- Currently only used by some extras/ scripts:
--- | Parse hledger CLI options from the command line using the given
--- cmdargs mode, and either return them or, if a help flag is present,
--- print the mode help and exit the program.
-getCliOpts :: Mode RawOpts -> IO CliOpts
-getCliOpts mode' = do
+-- | A helper for addon commands: this parses options and arguments from 
+-- the current command line using the given hledger-style cmdargs mode, 
+-- and returns a CliOpts. Or, with --help or -h present, it prints 
+-- long or short help, and exits the program. 
+-- When --debug is present, also prints some debug output.
+--
+-- The help texts are generated from the mode.
+-- Long help includes the full usage description generated by cmdargs
+-- (including all supported options), framed by whatever pre- and postamble
+-- text the mode specifies. It's intended that this forms a complete
+-- help document or manual.
+--
+-- Short help is a truncated version of the above: the preamble and
+-- the first part of the usage, up to the first line containing "flags:"
+-- (normally this marks the start of the common hledger flags);
+-- plus a mention of --help and the (presumed supported) common
+-- hledger options not displayed.
+--
+-- Tips:
+-- Empty lines in the pre/postamble are removed by cmdargs; 
+-- add a space character to preserve them.
+--
+getHledgerCliOpts :: Mode RawOpts -> IO CliOpts
+getHledgerCliOpts mode' = do
   args' <- getArgs
-  let rawopts = decodeRawOpts $ processValue mode' args'
+  let rawopts = either usageError decodeRawOpts $ process mode' args'
   opts <- rawOptsToCliOpts rawopts
   debugArgs args' opts
-  -- if any (`elem` args) ["--help","-h","-?"]
-  when ("h" `inRawOpts` rawopts_ opts) $ putStr (showModeUsage mode') >> exitSuccess
-  when ("help" `inRawOpts` rawopts_ opts) $ printHelpForTopic (topicForMode mode') >> exitSuccess
+  when ("help" `inRawOpts` rawopts_ opts) $ putStr longhelp  >> exitSuccess
+  when ("h"    `inRawOpts` rawopts_ opts) $ putStr shorthelp >> exitSuccess
   return opts
   where
+    longhelp = showModeUsage mode'
+    shorthelp =
+      unlines $
+        (reverse $ dropWhile null $ reverse $ takeWhile (not . ("flags:" `isInfixOf`)) $ lines longhelp)
+        ++
+        [""
+        ,"  See --help for full detail, including common hledger options."
+        ]
     -- | Print debug info about arguments and options if --debug is present.
     debugArgs :: [String] -> CliOpts -> IO ()
     debugArgs args' opts =
@@ -465,7 +549,7 @@ widthFromOpts :: CliOpts -> Int
 widthFromOpts CliOpts{width_=Nothing, available_width_=w} = w
 widthFromOpts CliOpts{width_=Just s}  =
     case runParser (read `fmap` some digitChar <* eof :: ParsecT Dec String Identity Int) "(unknown)" s of
-        Left e   -> optserror $ "could not parse width option: "++show e
+        Left e   -> usageError $ "could not parse width option: "++show e
         Right w  -> w
 
 -- for register:
@@ -483,7 +567,7 @@ registerWidthsFromOpts :: CliOpts -> (Int, Maybe Int)
 registerWidthsFromOpts CliOpts{width_=Nothing, available_width_=w} = (w, Nothing)
 registerWidthsFromOpts CliOpts{width_=Just s}  =
     case runParser registerwidthp "(unknown)" s of
-        Left e   -> optserror $ "could not parse width option: "++show e
+        Left e   -> usageError $ "could not parse width option: "++show e
         Right ws -> ws
     where
         registerwidthp :: (Stream s, Char ~ Token s) => ParsecT Dec s m (Int, Maybe Int)
@@ -516,35 +600,41 @@ defaultBalanceLineFormat = BottomAligned [
 
 -- Other utils
 
--- | Get the sorted unique precise names and display names of hledger
--- add-ons found in the current user's PATH. The precise names are the
--- add-on's filename with the "hledger-" prefix removed. The display
--- names have the file extension removed also, except when it's needed
--- for disambiguation.
+-- | Get the sorted unique canonical names of hledger addon commands
+-- found in the current user's PATH. These are used in command line
+-- parsing and to display the commands list.
 --
--- -- Also when there are exactly two similar names, one with the .hs or
--- -- .lhs extension and the other with the .exe extension or no
--- -- extension - presumably source and compiled versions of a haskell
--- -- script - we exclude the source version.
+-- Canonical addon names are the filenames of hledger-* executables in
+-- PATH, without the "hledger-" prefix, and without the file extension
+-- except when it's needed for disambiguation (see below).
 --
--- This function can return add-on names which shadow built-in command
--- names, but hledger will ignore these.
---
-hledgerAddons :: IO ([String],[String])
+-- When there are exactly two versions of an executable (same base
+-- name, different extensions) that look like a source and compiled
+-- pair (one has .exe, .com, or no extension), the source version will
+-- be excluded (even if it happens to be newer). When there are three
+-- or more versions (or two versions that don't look like a
+-- source/compiled pair), they are all included, with file extensions
+-- intact.
+-- 
+hledgerAddons :: IO [String]
 hledgerAddons = do
-  exes <- hledgerExecutablesInPath
-  let precisenames = -- concatMap dropRedundant $
-                     -- groupBy (\a b -> dropExtension a == dropExtension b) $
-                     map stripprefix exes
-  let displaynames = concatMap stripext $
-                     groupBy (\a b -> dropExtension a == dropExtension b) precisenames
-  return (precisenames, displaynames)
-  where
-    stripprefix = drop (length progname + 1)
-    -- dropRedundant [f,f2] | takeExtension f `elem` ["",".exe"] && takeExtension f2 `elem` [".hs",".lhs"] = [f]
-    -- dropRedundant fs = fs
-    stripext [f] = [dropExtension f]
-    stripext fs  = fs
+  -- past bug generator
+  as1 <- hledgerExecutablesInPath                                  -- ["hledger-check","hledger-check-dates","hledger-check-dates.hs","hledger-check.hs","hledger-check.py"]
+  let as2 = map stripPrognamePrefix as1                            -- ["check","check-dates","check-dates.hs","check.hs","check.py"]
+  let as3 = sortBy (comparing takeBaseName) as2                    -- ["check","check.hs","check.py","check-dates","check-dates.hs"]
+  let as4 = groupBy (\a b -> takeBaseName a == takeBaseName b) as3 -- [["check","check.hs","check.py"],["check-dates","check-dates.hs"]]
+  let as5 = concatMap dropRedundantSourceVersion as4               -- ["check","check.hs","check.py","check-dates"]
+  return as5
+
+stripPrognamePrefix = drop (length progname + 1)
+
+dropRedundantSourceVersion [f,g]
+  | takeExtension f `elem` compiledExts = [f]
+  | takeExtension g `elem` compiledExts = [g]
+dropRedundantSourceVersion fs = fs
+
+compiledExts = ["",".com",".exe"] 
+
 
 -- | Get the sorted unique filenames of all hledger-* executables in
 -- the current user's PATH. Currently these are: files in any of the
