@@ -10,6 +10,7 @@ module Hledger.Cli.Print (
   printmode
  ,print'
  ,entriesReportAsText
+ ,originalTransaction
  ,tests_Hledger_Cli_Print
 )
 where
@@ -27,14 +28,16 @@ import Hledger.Cli.Add ( transactionsSimilarTo )
 
 
 printmode = (defCommandMode $ ["print"] ++ aliases) {
-  modeHelp = "show transaction entries" `withAliases` aliases
+  modeHelp = "show transaction journal entries" `withAliases` aliases
  ,modeGroupFlags = Group {
      groupUnnamed = [
         let matcharg = "STR"
         in
          flagReq  ["match","m"] (\s opts -> Right $ setopt "match" s opts) matcharg
          ("show the transaction whose description is most similar to "++matcharg
-          ++ ", and is most recent")
+          ++ ", and is most recent"),
+        flagNone ["explicit","x"] (setboolopt "explicit")
+         "show all amounts explicitly"
         ]
         ++ outputflags
     ,groupHidden = []
@@ -43,6 +46,17 @@ printmode = (defCommandMode $ ["print"] ++ aliases) {
  }
   where aliases = []
 
+showTransaction' :: CliOpts -> Transaction -> String
+showTransaction' opts
+    | boolopt "explicit" $ rawopts_ opts = showTransactionUnelided
+    | otherwise = showTransactionUnelided . originalTransaction
+
+originalTransaction :: Transaction -> Transaction
+originalTransaction t = t { tpostings = map originalPosting' $ tpostings t } where
+    -- We don't want plain original postings because print wouldn't issue alias
+    -- directives. Thus we are going to print effective account name.
+    originalPosting' p = (originalPosting p) { paccount = paccount p }
+
 -- | Print journal transactions in standard format.
 print' :: CliOpts -> Journal -> IO ()
 print' opts j = do
@@ -57,12 +71,15 @@ printEntries opts@CliOpts{reportopts_=ropts} j = do
       fmt = outputFormatFromOpts opts
       (render, ropts') = case fmt of
         "csv" -> ((++"\n") . printCSV . entriesReportAsCsv, ropts{accountlistmode_=ALFlat})
-        _     -> (entriesReportAsText,                      ropts)
+        _     -> (entriesReportAsText' opts,                ropts)
   writeOutput opts $ render $ entriesReport ropts' q j
 
 entriesReportAsText :: EntriesReport -> String
 entriesReportAsText items = concatMap showTransactionUnelided items
 
+entriesReportAsText' :: CliOpts -> EntriesReport -> String
+entriesReportAsText' = concatMap . showTransaction'
+
 -- XXX
 -- tests_showTransactions = [
 --   "showTransactions" ~: do
@@ -106,7 +123,7 @@ entriesReportAsText items = concatMap showTransactionUnelided items
 
 entriesReportAsCsv :: EntriesReport -> CSV
 entriesReportAsCsv txns =
-  ["txnidx","date","date2","status","code","description","comment","account","amount","commodity","credit","debit","status","posting-comment"] :
+  ["txnidx","date","date2","status","code","description","comment","account","amount","commodity","credit","debit","posting-status","posting-comment"] :
   concatMap transactionToCSV txns
 
 -- | Generate one CSV record per posting, duplicating the common transaction fields.
@@ -131,7 +148,7 @@ postingToCSV p =
     let amount = showAmount a_ in
     let commodity = T.unpack c in
     let credit = if q < 0 then showAmount $ negate a_ else "" in
-    let debit  = if q > 0 then showAmount a_ else "" in
+    let debit  = if q >= 0 then showAmount a_ else "" in
     account:amount:commodity:credit:debit:status:comment:[])
    amounts
   where
