@@ -4,6 +4,8 @@ Print some statistics for the journal.
 
 -}
 
+{-# LANGUAGE OverloadedStrings #-}
+
 module Hledger.Cli.Stats (
   statsmode
  ,stats
@@ -14,7 +16,8 @@ import Data.List
 import Data.Maybe
 import Data.Ord
 import Data.HashSet (size, fromList)
-import Data.Text (pack)
+-- import Data.Text (Text)
+import qualified Data.Text as T
 import Data.Time.Calendar
 import System.Console.CmdArgs.Explicit
 import Text.Printf
@@ -30,7 +33,7 @@ statsmode = (defCommandMode $ ["stats"] ++ aliases) {
   modeHelp = "show some journal statistics" `withAliases` aliases
  ,modeGroupFlags = Group {
      groupUnnamed = [
-        flagReq  ["output-file","o"]   (\s opts -> Right $ setopt "output-file" s opts) "FILE[.FMT]" "write output to FILE instead of stdout. A recognised FMT suffix influences the format."
+        flagReq  ["output-file","o"]   (\s opts -> Right $ setopt "output-file" s opts) "FILE" "write output to FILE.  A file extension matching one of the above formats selects that format."
         ]
     ,groupHidden = []
     ,groupNamed = [generalflagsgroup1]
@@ -46,7 +49,7 @@ stats opts@CliOpts{reportopts_=reportopts_} j = do
   let q = queryFromOpts d reportopts_
       l = ledgerFromJournal q j
       reportspan = (ledgerDateSpan l) `spanDefaultsFrom` (queryDateSpan False q)
-      intervalspans = splitSpan (intervalFromOpts reportopts_) reportspan
+      intervalspans = splitSpan (interval_ reportopts_) reportspan
       showstats = showLedgerStats l d
       s = intercalate "\n" $ map showstats intervalspans
   writeOutput opts s
@@ -60,16 +63,16 @@ showLedgerStats l today span =
       w1 = maximum $ map (length . fst) stats
       -- w2 = maximum $ map (length . show . snd) stats
       stats = [
-         ("Main journal file" :: String, path) -- ++ " (from " ++ source ++ ")")
-        ,("Included journal files", unlines $ drop 1 $ journalFilePaths j)
+         ("Main file" :: String, path) -- ++ " (from " ++ source ++ ")")
+        ,("Included files", unlines $ drop 1 $ journalFilePaths j)
         ,("Transactions span", printf "%s to %s (%d days)" (start span) (end span) days)
         ,("Last transaction", maybe "none" show lastdate ++ showelapsed lastelapsed)
         ,("Transactions", printf "%d (%0.1f per day)" tnum txnrate)
         ,("Transactions last 30 days", printf "%d (%0.1f per day)" tnum30 txnrate30)
         ,("Transactions last 7 days", printf "%d (%0.1f per day)" tnum7 txnrate7)
-        ,("Payees/descriptions", show $ size $ fromList $ map (pack . tdescription) ts)
+        ,("Payees/descriptions", show $ size $ fromList $ map (tdescription) ts)
         ,("Accounts", printf "%d (depth %d)" acctnum acctdepth)
-        ,("Commodities", printf "%s (%s)" (show $ length cs) (intercalate ", " cs))
+        ,("Commodities", printf "%s (%s)" (show $ length cs) (T.intercalate ", " cs))
       -- Transactions this month     : %(monthtxns)s (last month in the same period: %(lastmonthtxns)s)
       -- Uncleared transactions      : %(uncleared)s
       -- Days since reconciliation   : %(reconcileelapsed)s
@@ -80,7 +83,7 @@ showLedgerStats l today span =
              path = journalFilePath j
              ts = sortBy (comparing tdate) $ filter (spanContainsDate span . tdate) $ jtxns j
              as = nub $ map paccount $ concatMap tpostings ts
-             cs = Map.keys $ canonicalStyles $ concatMap amounts $ map pamount $ concatMap tpostings ts
+             cs = Map.keys $ commodityStylesFromAmounts $ concatMap amounts $ map pamount $ concatMap tpostings ts
              lastdate | null ts = Nothing
                       | otherwise = Just $ tdate $ last ts
              lastelapsed = maybe Nothing (Just . diffDays today) lastdate
